# $Id: vCard.pm,v 1.7 2004/09/26 02:54:41 asc Exp $
use strict;

package XML::Generator::vCard;
use base qw (XML::SAX::Base);

$XML::Generator::vCard::VERSION = '1.0_01';

=head1 NAME

XML::Generator::vCard - generate SAX2 events for vCard 3.0

=head1 SYNOPSIS

 use XML::SAX::Writer;
 use XML::Generator::vCard;

 my $writer = XML::SAX::Writer->new();
 my $driver = XML::Generator::vCard->new(Handler=>$writer);

 $driver->parse_files("test.vcd");

=head1 DESCRIPTION

Generate SAX2 events for vCard 3.0

=cut

use Encode;
use MIME::Base64;
use Text::vCard;

# This on died in IETF draft land and was
# subsequently sucked into the vortex. I
# would like nothing more than some definitive
# vcard-xml thingy that I could point (please,
# don't say RDF; not yet anyway) so in the
# interim this will have to do...

# http://www.ietf.org/internet-drafts/ \
#  draft-dawson-vCard-xml-dtd-04.txt

use constant NS_VCARD  => "http://aaronland.info/perl/xml/generator/vcard#";
use constant NS_CUSTOM => "http://aaronland.info/perl/xml/generator/vcard/x-custom#";

use constant VCARD_VERSION => "3.0";

=head1 PACKAGE METHODS

=cut

=head2 __PACKAGE__->new(%args)

This method inherits from I<XML::SAX::Base>

=cut

=head1 OBJECT METHODS

=cut

sub parse {
  my $self = shift;
  my $str  = shift;

  return 1;
}

=head2 $pkg->parse_files(@files)

=cut

sub parse_files {
  my $self  = shift;
  my @files = @_;

  my $cards = undef;

  eval {
      $cards = Text::vCard->load(@files);
  };

  if ($@) {
      warn $@;
      return 0;
  }

  return $self->_render_doc($cards);
}

=head1 PRIVATE METHODS

Private methods are documented below in case you need to subclass
this package to tweak its output. For example, filtering out B<X-AB*>
that Apple's Address Book adds to its vCard export facility.

=cut

=head2 $obj->_render_doc(\@vcards)

=cut

sub _render_doc {
    my $self  = shift;
    my $cards = shift;

    $self->_start_document();

    if (scalar(@$cards) > 1) {
	
	$self->start_element({Name=>"vCardSet"});
	
	foreach my $vcard (@$cards) {
	    $self->_render_card($vcard);
	}
	
	$self->end_element({Name=>"vCardSet"});
    }
    
    else {
	$self->_render_card($cards->[0]);
    }
    
    #
    
    $self->_end_document();
    return 1;
}

=head2 $obj->_render_card(Text::vCard)

=cut

sub _render_card {
  my $self  = shift;
  my $vcard = shift;

  my $attrs = {
      "{}version" => {Name  => "version",
		      Value => ($vcard->version() || VCARD_VERSION)},
      "{}class"   => {Name  => "class",
		      Value => ($vcard->class() || "PUBLIC")},
  };

  #

  foreach my $prop ("uid","rev","prodid") {
      if (my $value = $vcard->$prop()) {
	  $attrs->{"{}$prop"} = {Name  => $prop,
				 Value => $value};
      }
  }

  $self->start_element({Name       => "vCard",
			Attributes => $attrs});

  # 

  $self->_render_fn($vcard);
  $self->_render_n($vcard);
  $self->_render_nickname($vcard);
  $self->_render_photo($vcard);
  $self->_render_bday($vcard);
  $self->_render_adrs($vcard);
  $self->_render_labels($vcard);
  $self->_render_tels($vcard);
  $self->_render_emails($vcard);
  $self->_render_ims($vcard);
  $self->_render_mailer($vcard);
  $self->_render_tz($vcard);
  $self->_render_geo($vcard);
  $self->_render_org($vcard);
  $self->_render_title($vcard);
  $self->_render_role($vcard);
  $self->_render_logo($vcard);
  # AGENT
  $self->_render_categories($vcard);
  $self->_render_note($vcard);
  # SORT
  $self->_render_sound($vcard);
  $self->_render_url($vcard);
  $self->_render_key($vcard);
  $self->_render_custom($vcard);

  $self->end_element({Name=>"vCard"});

  return 1;
}

=head2 $obj->_render_fn(Text::vCard)

=cut

sub _render_fn {
    my $self = shift;
    my $vcard = shift;

    $self->_pcdata({name  => "fn",
		    value => $vcard->fn()});
    
    return 1;
}

=head2 $obj->_render_n(Text::vCard)

=cut

sub _render_n {
    my $self = shift;
    my $vcard = shift;

    if (my $n = $vcard->name()) {
	
	if (($n->family()) || ($n->given())) {
	    $self->start_element({Name=>"n"});
	    
	    $self->_pcdata({name  => "family",
			    value => $n->family()});
	    
	    $self->_pcdata({name  => "given",
			    value => $n->given()});
	    
	    if (my $o = $n->additional()) {
		$self->_pcdata({name  => "other",
				value => $o});
	    }
	    
	    if (my $p = $n->prefixes()) {
		$self->_pcdata({name  => "prefix",
				value => $p});
	    }
	    
	    if (my $s = $n->suffixes()) {
		$self->_pcdata({name  => "suffix",
				value => $s});
	    }
	    
	    $self->end_element({Name=>"n"});
	}     
    }
    
    return 1;
}

=head2 $obj->_render_nickname(Text::vCard)

=cut

sub _render_nickname {
    my $self = shift;
    my $vcard = shift;

    if (my $nick = $vcard->nickname()) {
	$self->_pcdata({name  => "nickname",
			value => $nick});
    }

    return 1;
}

=head2 $obj->_render_photo(Text::vCard)

=cut

sub _render_photo {
    my $self = shift;
    my $vcard = shift;

    if (my $p = $vcard->photo()) {
	$self->_media({name   => "photo",
		       media  => $p});
    }
    
    return 1;
}


=head2 $obj->_render_bday(Text::vCard)

=cut

sub _render_bday {
    my $self = shift;
    my $vcard = shift;

    if (my $bday = $vcard->bday()) {
	$self->_pcdata({name  => "bday",
			value => $bday});
    }

    return 1;
}

=head2 $obj->_render_adrs(Text::vCard)

=cut

sub _render_adrs {
    my $self  = shift;
    my $vcard = shift;

    foreach my $adr ($vcard->addresses()) {
	
	if (! $adr) {
	    next;
	}
	
	my $types = join(";",$adr->types());
	
	$self->start_element({Name       => "adr",
			      Attributes => {"{}del.type"=>{Name  => "del.type",
							    Value => $types}}
			  });
	
	if (my $p = $adr->pobox()) {
	    $self->_pcdata({name  => "pobox",
			    value => $p});
	}
	
	if (my $e = $adr->extended()) {
	    $self->_pcdata({name  => "extadr",
			    value => $e});
	}
	
	if (my $s = $adr->street()) {
	    $self->_pcdata({name  => "street",
			    value => $s});
	}
	
	if (my $c = $adr->city()) {
	    $self->_pcdata({name  => "locality",
			    value => $c});
	}
	
	if (my $r = $adr->region()) {
	    $self->_pcdata({name  => "region",
			    value => $r});
	}
	
	if (my $p = $adr->post_code()) {
	    $self->_pcdata({name  => "pcode",
			    value => $p});
	}
	
	if (my $c = $adr->country()) {
	    $self->_pcdata({name  => "country",
			    value => $c});
	}
	
	$self->end_element({Name=>"adr"});
    }
    
    return 1;
}


=head2 $obj->_render_labels(Text::vCard)

=cut

sub _render_labels {
    my $self  = shift;
    my $vcard = shift;

    foreach my $l ($vcard->labels()) {
	next unless ($l);
	
	my $types = join(";",$l->types());
	
	$self->_pcdata({name  => "label",
			value => $l->value(),
			attrs => {"{}del.type"=>{Name  => "del.type",
						 Value => $types}}
		    });
    }
    
    return 1;
}

=head2 $obj->_render_tels(Text::vCard)

=cut

sub _render_tels {
    my $self  = shift;
    my $vcard = shift;

    foreach my $tel ($vcard->tels()) {
	
	if (! $tel) {
	    next;
	}
	
	my $types = join(";",$tel->types());
	
	$self->_pcdata({name  => "tel",
			value => $tel->value(),
			attrs => {"{}tel.type"=>{Name  => "tel.type",
						 Value => $types}}
		    });
    }
    
    return 1;
}

=head2 $obj->_render_emails(Text::vCard)

=cut

sub _render_emails {
    my $self  = shift;
    my $vcard = shift;

  foreach my $e ($vcard->emails()) {
      next unless ($e);

      my $types = join(";",$e->types());

      $self->_pcdata({name  => "email",
		      value => $e->value(),
		      attrs => {"{}email.type"=>{Name  => "email.type",
						 Value => $types}}
		  });
  }

    return 1;
}

sub _render_ims {
    my $self  = shift;
    my $vcard = shift;

    foreach my $im ($vcard->instantmessage_addresses()) {
	next unless ($im);
	
	my $types = join(";",$im->types());
	
	$self->_pcdata({name  => "x:instantmessage",
			value => $im->value(),
			attrs => {"{}im.type"=>{Name  => "im.type",
						Value => $types}}
		    });
    }
 
    return 1;
}

=head2 $obj->_render_mailer(Text::vCard)

=cut

sub _render_mailer {
    my $self  = shift;
    my $vcard = shift;

    if (my $m = $vcard->mailer()) {
	$self->_pcdata({name  => "mailer",
			value => $m});
    }

    return 1;
}

=head2 $obj->_render_tz(Text::vCard)

=cut

sub _render_tz {
    my $self  = shift;
    my $vcard = shift;

    if (my $tz = $vcard->tz()) {
	$self->_pcdata({name  => "tz",
			value => $tz});
    }

    return 1;
}

=head2 $obj->_render_geo(Text::vCard)

=cut

sub _render_geo {
    my $self  = shift;
    my $vcard = shift;

    if (my $g = $vcard->geo()) {
	$self->start_element({Name=>"geo"});
	$self->_pcdata({name  => "lat",
			value => $g->latitude()});
	$self->_pcdata({name  => "lon",
			value => $g->longitude()});
	$self->end_element({Name=>"geo"});
    }
    
    return 1;
}

=head2 $obj->_render_org(Text::vCard)

=cut

sub _render_org {
    my $self = shift;
    my $vcard = shift;

    if (my $org = $vcard->org()) {
	$self->start_element({Name=>"org"});
	$self->_pcdata({name  => "orgnam",
			value => $org});
	$self->end_element({Name=>"org"});
    }

    return 1;
}

=head2 $obj->_render_title(Text::vCard)

=cut

sub _render_title {
    my $self  = shift;
    my $vcard = shift;

    if (my $t = $vcard->title()) {
	$self->_pcdata({name  => "title",
			value => $t});
    }

    return 1;
}

=head2 $obj->_render_role(Text::vCard)

=cut

sub _render_role {
    my $self = shift;
    my $vcard = shift;

    if (my $r = $vcard->role()) {
	$self->_pcdata({name  => "role",
			value => $r});
    }

    return 1;
}

=head2 $obj->_render_logo(Text::vCard)

=cut

sub _render_logo {
    my $self = shift;
    my $vcard = shift;

    if (my $l = $vcard->logo()) {
	$self->_media({name  => "logo",
		       media => $l});
    }

    return 1;
}

=head2 $obj->_render_categories(Text::vCard)

=cut

sub _render_categories {
    my $self = shift;
    my $vcard = shift;

    if (my $c = $vcard->categories() || $vcard->category()) {
	
	$self->start_element({Name=>"categories"});
	
	foreach (split(",",$c)) {
	    $self->_pcdata({name  => "item",
			    value => $_});
	}
	
	$self->end_element({Name=>"categories"});
    }

    return 1;
}

=head2 $obj->_render_note(Text::vCard)

=cut

sub _render_note {
    my $self = shift;
    my $vcard = shift;

    if (my $n = $vcard->note()) {
	$self->_pcdata({name  => "note",
			cdata => 1,
			value => $n});	
    }
    
    return 1;
}

=head2 $self->_render_sound(Text::vCard)

=cut

sub _render_sound {
    my $self = shift;
    my $vcard = shift;

    if (my $s = $vcard->sound()) {
	$self->_media({name  => "sound",
		       media => $s});
    }
}

=head2 $self->_render_url(Text::vCard)

=cut

sub _render_url {
    my $self = shift;
    my $vcard = shift;

    if (my $url = $vcard->url()) {
	$self->_pcdata({name       => "url",
			Attributes => {"{}uri" => {Name  => "uri",
						   Value => $url}}});
    }
    
    return 1;
}

=head2 $obj->_render_key(Text::vCard)

=cut

sub _render_key {
    my $self = shift;
    my $vcard = shift;

    if (my $k = $vcard->key()) {
	$self->_media({name  => "key",
		       media => $k});
    }

    return 1;
}

=head2 $obj->_render_custom(Text::vCard)

=cut

sub _render_custom {
    my $self  = shift;
    my $vcard = shift;

    foreach my $c ($vcard->custom_fields()) {

	if (! $c) {
	    next;
	}

	my $field = lc($c->name());
	my %attrs = ();

	if (my $types = join(";",$c->types())) {
	    %attrs = ("{}im.type"=>{Name  => "type",
				    Value => $types});
	}

	$self->_pcdata({name  => "x:$field",
			value => $c->value(),
			attrs => \%attrs});
    }

    return 1;
}

sub _pcdata {
  my $self = shift;
  my $data = shift;
  $self->start_element({Name=>$data->{name},Attributes=>$data->{attrs}});
  $self->start_cdata() if ($data->{cdata});
  $self->characters({Data=>encode_utf8($data->{value})});
  $self->end_cdata() if ($data->{cdata});
  $self->end_element({Name=>$data->{name}});
  return 1;
}

sub _media {
  my $self = shift;
  my $data = shift;

  my $attrs = {};

  # as in not 'key' and not something pointing to an 'uri'

  if (($data->{name} !~ /^k/) && ($data->{type})) {

      # as in 'photo' or 'logo' 
      # and not 'sound'
      
      my $mime = ($data->{name} =~ /^[pl]/i) ? "img" : "aud";
      
      $attrs = {"{}$mime.type"=>{Name  => "$mime.type",
				 Value => $data->{type}}};
  }

  #

  my $obj = $data->{media};

  $self->start_element({Name       => $data->{name},
			Attributes => $attrs});

  if ($obj->is_binary()) {
      $self->_pcdata({name  => "b64bin",
		      value => encode_base64($obj->value()),
		      cdata => 1});
  }

  else {
      $self->_pcdata({name  => "extref",
		      attrs => {"{}uri" => {Name  => "uri",
					    Value => $obj->value()}}
		  });
  }

  $self->end_element({Name=>$data->{name}});
  return 1;
}

sub _start_document {
    my $self = shift;

    $self->start_document();
    $self->xml_decl({Version  => "1.0",
		     Encoding => "UTF-8"});
    
    $self->start_prefix_mapping({Prefix       => "",
				 NamespaceURI => NS_VCARD});
    
    $self->start_prefix_mapping({Prefix       => "x",
				 NamespaceURI => NS_CUSTOM});

    return 1;
}

sub _end_document {
    my $self = shift;

    $self->end_prefix_mapping({Prefix => "x"});
    $self->end_prefix_mapping({Prefix => ""});

    $self->end_document();
    return 1;
}

sub DESTROY {}

=head1 VERSION

1.0_01

=head1 DATE

$Date: 2004/09/26 02:54:41 $

=head1 AUTHOR

Aaron Straup Cope E<lt>ascope@cpan.orgE<gt>

=head1 SEE ALSO

L<Text::vCard>

http://www.ietf.org/rfc/rfc2426.txt

http://www.ietf.org/rfc/rfc2425.txt

http://www.globecom.net/ietf/draft/draft-dawson-vcard-xml-dtd-03.html

=head1 BUGS

Please report all bugs via http://rt.cpan.org

=head1 LICENSE

Copyright (c) 2002-2004, Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it 
under the same terms as Perl itself.

=cut

return 1;
